# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.sftpcopy.sftpcopy
import gocept.sftpcopy.testing
import os
import os.path
import shutil
import tempfile
import unittest


class EndToEndTest(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        server_dir = os.path.join(self.tmpdir, 'server')
        os.mkdir(server_dir)
        self.sftp = gocept.sftpcopy.testing.SFTPThread(
            'localhost', 8022, server_dir)
        self.sftp.start()

        self.config = dict(local_path=self.tmpdir, remote_path='/',
                           hostname='localhost', port=8022,
                           username='user', password='secret')

    def tearDown(self):
        self.sftp.stop()
        shutil.rmtree(self.tmpdir)

    def test_upload(self):
        os.mkdir(os.path.join(self.tmpdir, 'new'))
        f = open(os.path.join(self.tmpdir, 'new', 'foo'), 'w')
        f.write('contents')
        f.close()

        self.config['mode'] = 'upload'
        gocept.sftpcopy.sftpcopy.main(self.config)

        uploaded = os.path.join(self.tmpdir, 'server', 'foo')
        self.assertTrue(os.path.isfile(uploaded))
        self.assertEqual('contents', open(uploaded).read())

        self.assertEqual(0, len(os.listdir(os.path.join(self.tmpdir, 'new'))))
        self.assertEqual(1, len(os.listdir(os.path.join(self.tmpdir, 'cur'))))

    def test_download(self):
        f = open(os.path.join(self.tmpdir, 'server', 'foo'), 'w')
        f.write('contents')
        f.close()

        self.config['mode'] = 'download'
        gocept.sftpcopy.sftpcopy.main(self.config)

        downloaded = os.path.join(self.tmpdir, 'new', 'foo')
        self.assertTrue(os.path.isfile(downloaded))
        self.assertEqual('contents', open(downloaded).read())

        self.assertEqual(
            0, len(os.listdir(os.path.join(self.tmpdir, 'server'))))



# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.amqprun.handler
import gocept.amqprun.interfaces
import zope.component.zcml
import zope.interface


class Uploader(object):

    def __init__(self, hostname, port, username, password, remote_path):
        self.sftp = gocept.sftpcopy.SFTPCopy(
            '/dev/null', hostname, port, username, password, remote_path)

    def __call__(self, message):
        self.sftp.connect()
        self.sftp.uploadFileContents(
            self.determine_filename(message), message.body)
        self.sftp.close()

    def determine_filename(self, message):
        filename = message.header.headers and message.header.headers.get(
            'X-Filename')
        if not filename:
            filename = message.generate_filename('${routing_key}-${unique}')
        return filename


class ISFTPDirective(zope.interface.Interface):

    routing_key = zope.configuration.fields.Tokens(
        title=u"Routing key(s) to listen on",
        value_type=zope.schema.TextLine())

    queue_name = zope.schema.TextLine(title=u"Queue name")

    arguments = gocept.amqprun.interfaces.RepresentableDict(
        key_type=zope.schema.TextLine(),
        value_type=zope.schema.TextLine(),
        required=False)

    hostname = zope.schema.TextLine(title=u"Remote hostname")
    port = zope.schema.Int(title=u"Remote port")
    username = zope.schema.TextLine(title=u"Username")
    password = zope.schema.TextLine(title=u"Password")
    remote_path = zope.schema.TextLine(title=u"Path on remote machine")


def sftp_directive(
        _context, routing_key, queue_name,
        hostname, port, username, password, remote_path, arguments=None):
    uploader = Uploader(hostname, port, username, password, remote_path)
    handler = gocept.amqprun.handler.HandlerDeclaration(
        queue_name, routing_key, uploader, arguments)
    zope.component.zcml.utility(
        _context,
        component=handler,
        name=unicode('gocept.sftpcopy.amqp.%s' % queue_name))



# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

from StringIO import StringIO
import paramiko
import sftpserver.stub_sftp
import socket
import threading
import time


class SFTPServer(sftpserver.stub_sftp.StubServer):

    username = 'user'
    password = 'secret'

    def get_allowed_auths(self, username):
        return 'password'

    def check_auth_password(self, username, password):
        if username == self.username and password == self.password:
            return paramiko.AUTH_SUCCESSFUL
        return paramiko.AUTH_FAILED


class SFTPThread(threading.Thread):

    def __init__(self, host, port, directory):
        self.host = host
        self.port = port
        self.directory = directory
        super(SFTPThread, self).__init__()
        self.daemon = True

    def run(self):
        self.running = True

        # I'd rather use fs.expose.sftp, since it has a much cleaner API, but
        # since it doesn't work at all in lots of obscure ways, I've hacked
        # this together from sftpserver

        sftpserver.stub_sftp.StubSFTPServer.ROOT = self.directory

        server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
        server_socket.settimeout(0.1)
        server_socket.bind((self.host, self.port))
        server_socket.listen(10)

        while self.running:
            try:
                conn, addr = server_socket.accept()
            except socket.timeout:
                continue

            transport = paramiko.Transport(conn)
            transport.add_server_key(DEFAULT_HOST_KEY)
            transport.set_subsystem_handler(
                'sftp', paramiko.SFTPServer,
                sftpserver.stub_sftp.StubSFTPServer)

            transport.start_server(server=SFTPServer())
            while transport.is_active():
                time.sleep(0.01)

    def stop(self):
        self.running = False
        self.join()


DEFAULT_HOST_KEY = paramiko.RSAKey.from_private_key(StringIO("-----BEGIN RSA PRIVATE KEY-----\nMIICXgIBAAKCAIEAl7sAF0x2O/HwLhG68b1uG8KHSOTqe3Cdlj5i/1RhO7E2BJ4B\n3jhKYDYtupRnMFbpu7fb21A24w3Y3W5gXzywBxR6dP2HgiSDVecoDg2uSYPjnlDk\nHrRuviSBG3XpJ/awn1DObxRIvJP4/sCqcMY8Ro/3qfmid5WmMpdCZ3EBeC0CAwEA\nAQKCAIBSGefUs5UOnr190C49/GiGMN6PPP78SFWdJKjgzEHI0P0PxofwPLlSEj7w\nRLkJWR4kazpWE7N/bNC6EK2pGueMN9Ag2GxdIRC5r1y8pdYbAkuFFwq9Tqa6j5B0\nGkkwEhrcFNBGx8UfzHESXe/uE16F+e8l6xBMcXLMJVo9Xjui6QJBAL9MsJEx93iO\nzwjoRpSNzWyZFhiHbcGJ0NahWzc3wASRU6L9M3JZ1VkabRuWwKNuEzEHNK8cLbRl\nTyH0mceWXcsCQQDLDEuWcOeoDteEpNhVJFkXJJfwZ4Rlxu42MDsQQ/paJCjt2ONU\nWBn/P6iYDTvxrt/8+CtLfYc+QQkrTnKn3cLnAkEAk3ixXR0h46Rj4j/9uSOfyyow\nqHQunlZ50hvNz8GAm4TU7v82m96449nFZtFObC69SLx/VsboTPsUh96idgRrBQJA\nQBfGeFt1VGAy+YTLYLzTfnGnoFQcv7+2i9ZXnn/Gs9N8M+/lekdBFYgzoKN0y4pG\n2+Q+Tlr2aNlAmrHtkT13+wJAJVgZATPI5X3UO0Wdf24f/w9+OY+QxKGl86tTQXzE\n4bwvYtUGufMIHiNeWP66i6fYCucXCMYtx6Xgu2hpdZZpFw==\n-----END RSA PRIVATE KEY-----\n"))



# Copyright (c) 2012 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.amqprun.testing
import os
import os.path
import shutil
import tempfile
import time


class IntegrationTest(gocept.amqprun.testing.MainTestCase):

    def setUp(self):
        super(IntegrationTest, self).setUp()
        self.tmpdir = tempfile.mkdtemp()
        self.sftp = gocept.sftpcopy.testing.SFTPThread(
            'localhost', 8022, self.tmpdir)
        self.sftp.start()

    def tearDown(self):
        self.sftp.stop()
        shutil.rmtree(self.tmpdir)
        super(IntegrationTest, self).tearDown()

    def test_message_contents_should_be_uploaded(self):
        self.make_config(
            __name__, 'upload', dict(
                routing_key='test.data',
                queue_name=self.get_queue_name('test'),
                hostname='localhost', port='8022',
                username='user', password='secret',
                remote_path='/'
                ))
        self.start_server()
        body = 'This is only a test.'
        self.send_message(body, routing_key='test.data')
        self.wait_for_processing()
        time.sleep(1)

        files = os.listdir(self.tmpdir)
        self.assertEqual(1, len(files))
        uploaded = os.path.join(self.tmpdir, files[0])
        self.assertTrue(os.path.isfile(uploaded))
        self.assertEqual('This is only a test.', open(uploaded).read())



# Copyright (c) 2007 gocept gmbh & co. kg
# See also LICENSE.txt

from setuptools import setup, find_packages
import os.path


setup(
    name='gocept.sftpcopy',
    version='0.2',
    author="Christian Zagrodnick <cz at gocept dot com>",
    author_email="cz@gocept.com",
    description="Upload/download files via SFTP to a  maildir structure",
    long_description=(
        open(os.path.join(os.path.dirname(__file__), 'README.txt')).read()
        + '\n\n'
        + open(os.path.join(os.path.dirname(__file__), 'CHANGES.txt')).read()
        + '\n\n'
        ),
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Operating System :: Unix',
    ],

    license="ZPL 2.1",
    url='https://code.gocept.com/hg/public/gocept.sftpcopy',

    packages=find_packages('src'),
    package_dir={'': 'src'},

    include_package_data=True,
    zip_safe=False,

    namespace_packages=['gocept'],
    install_requires=[
        'setuptools',
        'gocept.filestore',
        'paramiko',
    ],
    extras_require=dict(
        amqp=[
            'zope.component[zcml]',
            'zope.configuration',
            'zope.interface',
            'gocept.amqprun>=0.6dev',
        ],
        test=[
            'sftpserver',
        ]),
    entry_points=dict(console_scripts=[
            'sftpcopy=gocept.sftpcopy.sftpcopy:main']),
)
